@@ -2,9 +2,9 @@
 //
 // Copyright (C) Nicolas Peugnet <nicolas@club1.fr>
 //
-// This program is free software: you can redistribute it and/or modify
+// This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation, either version 3 of the License, or
+// the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
 //
 // This program is distributed in the hope that it will be useful,
@@ -12,8 +12,11 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
 //
-// You should have received a copy of the GNU General Public License
-// along with this program.  If not, see <https://www.gnu.org/licenses/>.
+// You should have received a copy of the GNU General Public License along
+// with this program; if not, see <https://www.gnu.org/licenses/>.
+
+//go:generate wget -nv https://www.debian.org/debian.css -O assets/debian.css
+//go:generate patch assets/debian.css assets/debian.css.diff
 
 package main
 
@@ -31,9 +34,11 @@ import (
 	"path"
 	"path/filepath"
 	"sort"
+	"strconv"
 	"strings"
 	"sync"
 	"syscall"
+	text_template "text/template"
 	"time"
 
 	"salsa.debian.org/lintian/lintian-ssg/ioutil"
@@ -42,6 +47,10 @@ import (
 	"salsa.debian.org/lintian/lintian-ssg/version"
 )
 
+const (
+	renderTagWorkers = 16
+)
+
 type tmplParams struct {
 	DateYear       int
 	DateHuman      string
@@ -69,6 +78,10 @@ type tagTmplParams struct {
 	PrevName string
 }
 
+type templateExecuter interface {
+	Execute(wr io.Writer, data any) error
+}
+
 var (
 	//go:embed templates/*.tmpl
 	tmplFS      embed.FS
@@ -79,6 +92,9 @@ var (
 	manualTmpl  = template.Must(template.Must(indexTmpl.Clone()).ParseFS(tmplDir, "manual.html.tmpl"))
 	aboutTmpl   = template.Must(template.Must(indexTmpl.Clone()).ParseFS(tmplDir, "about.html.tmpl"))
 	e404Tmpl    = template.Must(template.Must(indexTmpl.Clone()).ParseFS(tmplDir, "404.html.tmpl"))
+	taglistTmpl = text_template.Must(text_template.ParseFS(tmplDir, "taglist.js.tmpl"))
+	//go:embed assets/debian.css
+	debianCSS []byte
 	//go:embed assets/main.css
 	mainCSS []byte
 	//go:embed assets/openlogo-50.svg
@@ -92,14 +108,15 @@ var (
 var (
 	flagBaseURL    string
 	flagFooter     string
-	flagHelp       bool
+	flagLintianVer string
 	flagManualFile string
 	flagNoSitemap  bool
 	flagOutDir     string
 	flagStats      bool
 	flagTagsFile   string
-	flagVersion    bool
 	flagDebug      bool
+	flagHelp       bool
+	flagVersion    bool
 )
 
 const (
@@ -114,7 +131,7 @@ func usage() {
 	} else {
 		output = flag.CommandLine.Output()
 	}
-	fmt.Fprintf(output, `Usage: lintian-ssg [OPTIONS]...
+	fmt.Fprintf(output, `Usage: lintian-ssg [OPTION]...
 
 Generate a static web site for Lintian tags' explanations. By default, it will
 call lintian-explain-tags in a subprocess, except if the --tags-file option is
@@ -128,6 +145,7 @@ Options:
   --footer=TEXT         TEXT to add to the footer, inline Markdown elements will
                         be parsed.
 
+  --lintian-version=V   Override Lintian's version in output with V.
   --manual-file=FILE    Read Lintian's manual from FILE.
                         (default %q)
 
@@ -137,9 +155,10 @@ Options:
 
   --stats               Display some statistics.
   --tags-file=FILE      Read Lintian tags from FILE (in JSON format).
+
+  --debug               Print stack traces on errors.
   -h, --help            Show this help and exit.
   --version             Show version and exit.
-  --debug               Print stack traces on errors.
 `,
 		flagManualFileDef,
 		flagOutDirDef,
@@ -164,8 +183,7 @@ func createTagFile(name string) (page string, file *os.File, err error) {
 	return
 }
 
-func renderTag(tag *lintian.Tag, params *tmplParams, pages chan<- string, wg *sync.WaitGroup) {
-	defer wg.Done()
+func renderTag(tag *lintian.Tag, params *tmplParams, pages chan<- string) {
 	page, file, err := createTagFile(tag.Name)
 	if err != nil {
 		panic(err)
@@ -200,6 +218,7 @@ func writeAssets() error {
 		name    string
 		content io.Reader
 	}{
+		{"debian.css", bytes.NewReader(debianCSS)},
 		{"main.css", bytes.NewReader(mainCSS)},
 		{"openlogo-50.svg", bytes.NewReader(logoSVG)},
 		{"favicon.ico", bytes.NewReader(faviconICO)},
@@ -251,7 +270,7 @@ func writeManual(params *tmplParams, path string, pages chan<- string) error {
 	return ioutil.WriteFile(flagOutDir, path, &out)
 }
 
-func writeSimplePage(tmpl *template.Template, params any, path string, pages chan<- string) error {
+func writeSimplePage(tmpl templateExecuter, params any, path string, pages chan<- string) error {
 	file, err := os.Create(filepath.Join(flagOutDir, path))
 	if err != nil {
 		return err
@@ -282,26 +301,48 @@ func withRoot(params tmplParams, root string) tmplParams {
 	return params
 }
 
+func buildDate() time.Time {
+	source_date_epoch := os.Getenv("SOURCE_DATE_EPOCH")
+	if source_date_epoch == "" {
+		return start.UTC()
+	} else {
+		sde, err := strconv.ParseInt(source_date_epoch, 10, 64)
+		if err != nil {
+			panic(fmt.Sprintf("invalid SOURCE_DATE_EPOCH: %s", err))
+		}
+		return time.Unix(sde, 0).UTC()
+	}
+}
+
 func checkErr(err error, msg ...string) {
 	if err != nil {
 		panic(strings.Join(append(msg, err.Error()), ": "))
 	}
 }
 
+func checkPanic() {
+	if !flagDebug {
+		if p := recover(); p != nil {
+			log.Fatalln("ERROR:", p)
+		}
+	}
+}
+
 func Run() {
 	log.SetFlags(0)
 	flag.StringVar(&flagBaseURL, "base-url", "", "")
 	flag.StringVar(&flagFooter, "footer", "", "")
-	flag.BoolVar(&flagHelp, "h", false, "")
-	flag.BoolVar(&flagHelp, "help", false, "")
+	flag.StringVar(&flagLintianVer, "lintian-version", "", "")
 	flag.StringVar(&flagManualFile, "manual-file", flagManualFileDef, "")
 	flag.BoolVar(&flagNoSitemap, "no-sitemap", false, "")
 	flag.StringVar(&flagOutDir, "o", flagOutDirDef, "")
 	flag.StringVar(&flagOutDir, "output-dir", flagOutDirDef, "")
 	flag.BoolVar(&flagStats, "stats", false, "")
 	flag.StringVar(&flagTagsFile, "tags-file", "", "")
-	flag.BoolVar(&flagVersion, "version", false, "")
 	flag.BoolVar(&flagDebug, "debug", false, "")
+	flag.BoolVar(&flagHelp, "h", false, "")
+	flag.BoolVar(&flagHelp, "help", false, "")
+	flag.BoolVar(&flagVersion, "version", false, "")
 	flag.Usage = usage
 	flag.Parse()
 
@@ -334,31 +375,47 @@ func Run() {
 	}
 	checkErr(err, "get tags")
 
-	date := start.UTC()
+	date := buildDate()
 	params := tmplParams{
-		BaseURL:     flagBaseURL,
-		DateYear:    date.Year(),
-		DateHuman:   date.Format(time.RFC1123),
-		DateMachine: date.Format(time.RFC3339),
-		Version:     version.Number,
-		FooterHTML:  markdown.ToHTML(flagFooter, markdown.StyleInline),
+		BaseURL:        flagBaseURL,
+		DateYear:       date.Year(),
+		DateHuman:      date.Format(time.RFC1123),
+		DateMachine:    date.Format(time.RFC3339),
+		Version:        version.Number,
+		VersionLintian: flagLintianVer,
+		FooterHTML:     markdown.ToHTML(flagFooter, markdown.StyleInline),
 	}
 
 	tagList := make([]string, 0, 2048)
+	tagChan := make(chan *lintian.Tag, 32)
 	tagsWG := sync.WaitGroup{}
+	// Launch a limited amount of workers
+	for i := 0; i < renderTagWorkers; i++ {
+		go func() {
+			defer checkPanic()
+			for tag := range tagChan {
+				renderTag(tag, &params, pagesChan)
+				tagsWG.Done()
+			}
+		}()
+	}
 	err = tagsProvider.ForEach(func(tag *lintian.Tag) {
 		if params.VersionLintian == "" {
 			params.VersionLintian = tag.LintianVersion
 		}
+		// Override tag.LintianVersion in case --lintian-version option was passed
+		tag.LintianVersion = params.VersionLintian
 		tagsWG.Add(1)
-		go renderTag(tag, &params, pagesChan, &tagsWG)
+		tagChan <- tag
 		tagList = append(tagList, tag.Name)
 	})
 	checkErr(err, "iterate over tags")
+	close(tagChan)
 
 	tagListJSON, err := json.Marshal(tagList)
 	checkErr(err, "marshal tagList")
-	checkErr(ioutil.WriteFile(flagOutDir, "taglist.json", bytes.NewReader(tagListJSON)), "write taglist")
+	taglistParams := struct{ TagListJSON []byte }{tagListJSON}
+	checkErr(writeSimplePage(taglistTmpl, taglistParams, "taglist.js", nil), "write taglist.js")
 	checkErr(writeAssets(), "write assets")
 	checkErr(writeManual(&params, "manual/index.html", pagesChan), "write manual")
 	indexParams := indexTmplParams{withRoot(params, "./"), tagList}
@@ -398,12 +455,6 @@ total duration: %v
 }
 
 func main() {
-	defer func() {
-		if !flagDebug {
-			if p := recover(); p != nil {
-				log.Fatalln("ERROR:", p)
-			}
-		}
-	}()
+	defer checkPanic()
 	Run()
 }
